from bluer_objects import README

from bluer_ugv.README.consts import bluer_ugv_assets
from bluer_ugv.README.arzhang.consts import arzhang_assets2
from bluer_ugv.README.eagle.consts import eagle_assets2
from bluer_ugv.README.rangin.consts import rangin_assets2
from bluer_ugv.README.ravin.ravin3.consts import assets as ravin3_assets2
from bluer_ugv.README.ravin.consts import description as ravin_description
from bluer_ugv.README.swallow.consts import swallow_assets2

items = README.Items(
    [
        {
            "name": "swallow",
            "marquee": f"{swallow_assets2}/20250913_203635~2_1.gif?raw=true",
            "description": "based on power wheels.",
            "url": "./bluer_ugv/docs/swallow",
        },
        {
            "name": "arzhang",
            "marquee": f"{arzhang_assets2}/VID-20250905-WA0014_1.gif?raw=true",
            "description": "[swallow](./bluer_ugv/docs/swallow)'s little sister.",
            "url": "./bluer_ugv/docs/arzhang",
        },
        {
            "name": "rangin",
            "marquee": f"{rangin_assets2}/concepts/11.png",
            "description": "[swallow](./bluer_ugv/docs/swallow)'s ad robot.",
            "url": "./bluer_ugv/docs/rangin",
        },
        {
            "name": "ravin",
            "marquee": f"{ravin3_assets2}/20250723_095155~2_1.gif?raw=true",
            "description": ravin_description,
            "url": "./bluer_ugv/docs/ravin",
        },
        {
            "name": "eagle",
            "marquee": f"{eagle_assets2}/file_0000000007986246b45343b0c06325dd.png?raw=true",
            "description": "a remotely controlled ballon.",
            "url": "./bluer_ugv/docs/eagle",
        },
        {
            "name": "fire",
            "marquee": f"{bluer_ugv_assets}/fire.png?raw=true",
            "description": "based on a used car.",
            "url": "./bluer_ugv/docs/fire",
        },
        {
            "name": "beast",
            "marquee": "https://github.com/waveshareteam/ugv_rpi/raw/main/media/UGV-Rover-details-23.jpg",
            "description": "based on [UGV Beast PI ROS2](https://www.waveshare.com/wiki/UGV_Beast_PI_ROS2).",
            "url": "./bluer_ugv/docs/beast",
        },
    ]
)



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_ugv import env

review_args = [
    "[--frame_count <-1>]",
    "[--gif 0]",
    "[--rm_blank 0]",
]


def help_review(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~download,upload", mono=mono)

    args = review_args

    return show_usage(
        [
            "@swallow",
            "ultrasonic",
            "review",
            f"[{options}]",
            "[.|<object-name>]",
        ]
        + args,
        "review ultrasonic sensor data.",
        mono=mono,
    )


def help_test(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~upload", mono=mono)

    args = sorted(
        review_args
        + [
            "[--export 0]",
            "[--log 0]",
            "[--max_m {:.2f}]".format(env.BLUER_UGV_ULTRASONIC_SENSOR_MAX_M),
        ]
    )

    return show_usage(
        [
            "@swallow",
            "ultrasonic",
            "test",
            f"[{options}]",
            "[-|<object-name>]",
        ]
        + args,
        "test ultrasonic sensors.",
        mono=mono,
    )


help_functions = {
    "review": help_review,
    "test": help_test,
}



from bluer_objects.README.items import ImageItems
from bluer_objects.README.consts import assets

from bluer_ugv.README.swallow.consts import swallow_assets2

docs = [
    {
        "path": "../docs/swallow/digital/algo/yolo",
        "items": ImageItems(
            {
                f"{swallow_assets2}/yolo-debug.png": "",
            }
        ),
    },
    {
        "path": "../docs/swallow/digital/algo/yolo/train.md",
        "items": ImageItems(
            {
                f"{swallow_assets2}/yolo-debug-{0}.png": "",
                f"{swallow_assets2}/yolo-debug-{2}.png": "",
                f"{swallow_assets2}/yolo-debug-{3}.png": "",
                f"{swallow_assets2}/yolo-debug-{4}.png": "",
                f"{swallow_assets2}/yolo-debug-{5}.png": "",
                f"{assets}/swallow-debug-2025-09-16-19-53-19-4yzsp8/swallow-debug-2025-09-16-19-53-19-4yzsp8-2.gif": "",
            }
        ),
    },
]



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.eagle.consts import eagle_assets2

items = ImageItems(
    {
        f"{eagle_assets2}/file_0000000007986246b45343b0c06325dd.png": "",
        f"{eagle_assets2}/20250727_182113.jpg": "",
        f"{eagle_assets2}/20250726_171953.jpg": "",
    }
)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_ugv import NAME
from bluer_ugv.swallow.session.classical.keyboard.testing import test
from bluer_ugv.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="test",
)
parser.add_argument(
    "--keys",
    type=str,
    default=" ",
)
args = parser.parse_args()

success = False
if args.task == "test":
    success = test(args.keys)
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Dict, Tuple

from bluer_sbc.env import BLUER_SBC_SWALLOW_HAS_FULL_KEYBOARD

from bluer_ugv.logger import logger


class ControlKeys:
    def __init__(
        self,
        is_numpad: bool = BLUER_SBC_SWALLOW_HAS_FULL_KEYBOARD == 0,
        log: bool = True,
    ):
        self.is_numpad = is_numpad

        if log:
            logger.info(
                "{}: {}".format(
                    self.__class__.__name__,
                    "numpad" if self.is_numpad else "full",
                )
            )

        self._keys: Dict[str, Tuple[str, str]] = {
            "ultrasonic off": ("n", "-"),
            "ultrasonic on": ("m", "+"),
            "debug off": ("v", "9"),
            "debug on": ("b", "7"),
            "mode = none": ("y", "5"),
            "mode = action": ("g", "1"),
            "mode = training": ("t", "3"),
            "special key": ("z", "."),
            "speed backward": ("s", "2"),
            "speed forward": ("w", "8"),
            "steer left": ("a", "4"),
            "steer right": ("d", "6"),
            "stop": (" ", "0"),
        }

    @staticmethod
    def as_table():
        keys = ControlKeys(log=False)
        table = keys._keys.copy()  # pylint: disable=protected-access

        for is_numpad in [False, True]:
            keys.is_numpad = is_numpad

            special_keys = keys.special_keys
            for key, event in special_keys.items():
                if event not in table:
                    table[event] = ["", ""]

                table[event][int(is_numpad)] = f"*{key}"

        return [
            "| event | full keyboard | numpad |",
            "|-|-|-|",
        ] + sorted(
            [
                " | ".join(["", event] + list(keys) + [""])
                for event, keys in table.items()
            ]
        )

    def get(self, event: str) -> str:
        return self._keys[event][int(self.is_numpad)]

    @property
    def special_keys(self) -> Dict[str, str]:
        return (
            {
                "7": "exit",
                "9": "shutdown",
                "5": "reboot",
                "1": "update",
            }
            if self.is_numpad
            else {
                "i": "exit",
                "o": "shutdown",
                "p": "reboot",
                "u": "update",
            }
        )



from bluer_objects.README.items import Items
from bluer_objects import markdown
from bluer_objects.README.consts import assets_url

from bluer_ugv.README.swallow.consts import (
    swallow_assets2,
    swallow_electrical_designs,
)

items = markdown.generate_table(
    Items(
        [
            {
                "name": "shield",
                "url": "./bluer_ugv/docs/swallow/digital/design/shield.md",
                "marquee": f"{swallow_electrical_designs}/digital.png?raw=true",
            },
            {
                "name": "UGVs",
                "url": "./bluer_ugv/docs/UGVs",
                "marquee": f"{swallow_assets2}/20250912_211652.jpg?raw=true",
            },
            {
                "name": "terraform",
                "url": "./bluer_ugv/docs/swallow/digital/design/terraform.md",
                "marquee": f"{swallow_assets2}/20250611_100917.jpg?raw=true",
            },
            {
                "name": "arzhang validations",
                "url": "./bluer_ugv/docs/arzhang/validation",
                "marquee": assets_url(
                    suffix="{object_name}/{object_name}.gif".format(
                        object_name="swallow-debug-2025-09-25-13-16-59-rnm7jd"
                    )
                ),
            },
        ]
    ),
    cols=2,
    log=False,
)



from typing import Dict, Tuple
import numpy as np
import time
from enum import Enum, auto

from bluer_objects.graphics.signature import add_signature

from bluer_ugv import env


class DetectionState(Enum):
    CLEAR = auto()
    WARNING = auto()
    DANGER = auto()

    @property
    def color_code(self) -> Tuple[int, int, int]:
        return (
            [0, 255, 0]
            if self == DetectionState.CLEAR
            else ([255, 255, 0] if self == DetectionState.WARNING else [255, 0, 0])
        )


class Detection:
    def __init__(
        self,
        side: str,
        detection: bool = False,
        reason: str = "",
        echo_detected: bool = False,
        pulse_ms: float = 0.0,
        distance_mm: float = 0.0,
    ) -> None:
        self.time = time.time()
        self.side = side

        self.detection = detection
        self.reason = reason

        self.echo_detected = echo_detected
        self.pulse_ms = pulse_ms
        self.distance_mm = distance_mm

    def as_image(
        self,
        height: int = 512,
        width: int = 256,
        max_m: float = env.BLUER_UGV_ULTRASONIC_SENSOR_MAX_M,
        sign: bool = True,
        line_width: int = 80,
    ) -> np.ndarray:
        image = np.zeros((height, width, 3), dtype=np.uint8)

        if not self.detection:
            image[:, :, :] = 0
        elif not self.echo_detected:
            image[:, :, :] = 64
        else:
            distance = max(
                min(
                    int(self.distance_mm / 1000 / max_m * height),
                    height,
                ),
                0,
            )

            color_code = self.state.color_code
            for channel in range(3):
                image[:, :, channel] = color_code[channel]

            image[height - distance :, :, :] = 128

        if sign:
            image = add_signature(
                image,
                header=[self.as_str(short=True)],
                line_width=line_width,
            )

        return image

    def as_dict(self) -> Dict:
        return {
            "side": self.side,
            "detection": self.detection,
            "reason": self.reason,
            "echo_detected": self.echo_detected,
            "pulse_ms": self.pulse_ms,
            "distance_mm": self.distance_mm,
        }

    def as_str(
        self,
        short: bool = False,
    ) -> str:
        if self.detection:
            return ("{}: {}: {}" if short else "{:8}: {}: {:7}").format(
                self.side,
                (
                    (
                        "{:.0f} mm".format(self.distance_mm)
                        if short
                        else "{:6.2f} ms == {:5.0f} mm".format(
                            self.pulse_ms,
                            self.distance_mm,
                        )
                    )
                    if self.echo_detected
                    else "no echo" if self.detection else "no detection"
                ),
                self.state.name.lower(),
            )

        return "{}: no detection{}".format(
            self.side,
            f" ({self.reason})" if self.reason else "",
        )

    @property
    def is_blank(self) -> bool:
        return not self.detection or not self.echo_detected

    @property
    def state(self) -> DetectionState:
        if not self.echo_detected or not self.detection:
            return DetectionState.CLEAR

        if self.distance_mm < env.BLUER_UGV_ULTRASONIC_SENSOR_DANGER_THRESHOLD:
            return DetectionState.DANGER

        if self.distance_mm < env.BLUER_UGV_ULTRASONIC_SENSOR_WARNING_THRESHOLD:
            return DetectionState.WARNING

        return DetectionState.CLEAR



NAME = "bluer_ugv"

ICON = "🐬"

DESCRIPTION = f"{ICON} AI x UGV."

VERSION = "7.212.1"

REPO_NAME = "bluer-ugv"

MARQUEE = (
    "https://github.com/kamangir/assets2/raw/main/arzhang/VID-20250905-WA0014_1.gif"
)


ALIAS = "@ugv"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.swallow.consts import swallow_electrical_designs

docs = [
    {
        "path": "../docs/swallow/analog",
        "items": ImageItems(
            {
                f"{swallow_electrical_designs}/analog.png": f"{swallow_electrical_designs}/analog.svg",
            }
        ),
    }
]
